=== modified file 'mandos-ctl'
--- mandos-ctl	2019-03-09 18:17:28 +0000
+++ mandos-ctl	2019-03-04 18:38:56 +0000
@@ -42,9 +42,6 @@
 import json
 import unittest
 import logging
-import io
-import tempfile
-import contextlib
 
 import dbus
 
@@ -281,8 +278,8 @@
         commands which want to operate on all clients at the same time
         can override this run() method instead."""
         self.mandos = mandos
-        for client, properties in clients.items():
-            self.run_on_one_client(client, properties)
+        for client in clients:
+            self.run_on_one_client(client)
 
 class PrintCmd(Command):
     """Abstract class for commands printing client details"""
@@ -294,18 +291,12 @@
                     "ApprovalDuration", "Checker", "ExtendedTimeout",
                     "Expires", "LastCheckerStatus")
     def run(self, mandos, clients):
-        print(self.output(clients.values()))
+        print(self.output(clients))
 
 class PropertyCmd(Command):
     """Abstract class for Actions for setting one client property"""
-    def run_on_one_client(self, client, properties):
+    def run_on_one_client(self, client):
         """Set the Client's D-Bus property"""
-        log.debug("D-Bus: %s:%s:%s.Set(%r, %r, %r)", busname,
-                  client.__dbus_object_path__,
-                  dbus.PROPERTIES_IFACE, client_interface,
-                  self.property, self.value_to_set
-                  if not isinstance(self.value_to_set, dbus.Boolean)
-                  else bool(self.value_to_set))
         client.Set(client_interface, self.property, self.value_to_set,
                    dbus_interface=dbus.PROPERTIES_IFACE)
 
@@ -323,7 +314,7 @@
     @value_to_set.setter
     def value_to_set(self, value):
         """When setting, convert value to a datetime.timedelta"""
-        self._vts = int(round(value.total_seconds() * 1000))
+        self._vts = string_to_delta(value).total_seconds() * 1000
 
 # Actual (non-abstract) command classes
 
@@ -332,11 +323,11 @@
         self.verbose = verbose
 
     def output(self, clients):
-        default_keywords = ("Name", "Enabled", "Timeout", "LastCheckedOK")
-        keywords = default_keywords
         if self.verbose:
             keywords = self.all_keywords
-        return str(self.TableOfClients(clients, keywords))
+        else:
+            keywords = ("Name", "Enabled", "Timeout", "LastCheckedOK")
+        return str(self.TableOfClients(clients.values(), keywords))
 
     class TableOfClients(object):
         tableheaders = {
@@ -428,36 +419,25 @@
         return value
 
 class IsEnabledCmd(Command):
-    def run_on_one_client(self, client, properties):
-        if self.is_enabled(client, properties):
+    def run_on_one_client(self, client):
+        if self.is_enabled(client):
             sys.exit(0)
         sys.exit(1)
-    def is_enabled(self, client, properties):
-        log.debug("D-Bus: %s:%s:%s.Get(%r, %r)", busname,
-                  client.__dbus_object_path__,
-                  dbus.PROPERTIES_IFACE, client_interface,
-                  "Enabled")
-        return bool(client.Get(client_interface, "Enabled",
-                               dbus_interface=dbus.PROPERTIES_IFACE))
+    def is_enabled(self, client):
+        return client.Get(client_interface, "Enabled",
+                          dbus_interface=dbus.PROPERTIES_IFACE)
 
 class RemoveCmd(Command):
-    def run_on_one_client(self, client, properties):
-        log.debug("D-Bus: %s:%s:%s.RemoveClient(%r)", busname,
-                  server_path, server_interface,
-                  str(client.__dbus_object_path__))
+    def run_on_one_client(self, client):
         self.mandos.RemoveClient(client.__dbus_object_path__)
 
 class ApproveCmd(Command):
-    def run_on_one_client(self, client, properties):
-        log.debug("D-Bus: %s:%s.Approve(True)",
-                  client.__dbus_object_path__, client_interface)
+    def run_on_one_client(self, client):
         client.Approve(dbus.Boolean(True),
                        dbus_interface=client_interface)
 
 class DenyCmd(Command):
-    def run_on_one_client(self, client, properties):
-        log.debug("D-Bus: %s:%s.Approve(False)",
-                  client.__dbus_object_path__, client_interface)
+    def run_on_one_client(self, client):
         client.Approve(dbus.Boolean(False),
                        dbus_interface=client_interface)
 
@@ -496,14 +476,6 @@
     property = "Host"
 
 class SetSecretCmd(PropertyCmd, ValueArgumentMixIn):
-    @property
-    def value_to_set(self):
-        return self._vts
-    @value_to_set.setter
-    def value_to_set(self, value):
-        """When setting, read data from supplied file object"""
-        self._vts = value.read()
-        value.close()
     property = "Secret"
 
 class SetTimeoutCmd(PropertyCmd, MillisecondsValueArgumentMixIn):
@@ -524,6 +496,26 @@
                              MillisecondsValueArgumentMixIn):
     property = "ApprovalDuration"
 
+def has_actions(options):
+    return any((options.enable,
+                options.disable,
+                options.bump_timeout,
+                options.start_checker,
+                options.stop_checker,
+                options.is_enabled,
+                options.remove,
+                options.checker is not None,
+                options.timeout is not None,
+                options.extended_timeout is not None,
+                options.interval is not None,
+                options.approved_by_default is not None,
+                options.approval_delay is not None,
+                options.approval_duration is not None,
+                options.host is not None,
+                options.secret is not None,
+                options.approve,
+                options.deny))
+
 def add_command_line_options(parser):
     parser.add_argument("--version", action="version",
                         version="%(prog)s {}".format(version),
@@ -555,11 +547,11 @@
                         help="Remove client")
     parser.add_argument("-c", "--checker",
                         help="Set checker command for client")
-    parser.add_argument("-t", "--timeout", type=string_to_delta,
+    parser.add_argument("-t", "--timeout",
                         help="Set timeout for client")
-    parser.add_argument("--extended-timeout", type=string_to_delta,
+    parser.add_argument("--extended-timeout",
                         help="Set extended timeout for client")
-    parser.add_argument("-i", "--interval", type=string_to_delta,
+    parser.add_argument("-i", "--interval",
                         help="Set checker interval for client")
     approve_deny_default = parser.add_mutually_exclusive_group()
     approve_deny_default.add_argument(
@@ -570,9 +562,9 @@
         "--deny-by-default", action="store_false",
         dest="approved_by_default",
         help="Set client to be denied by default")
-    parser.add_argument("--approval-delay", type=string_to_delta,
+    parser.add_argument("--approval-delay",
                         help="Set delay before client approve/deny")
-    parser.add_argument("--approval-duration", type=string_to_delta,
+    parser.add_argument("--approval-duration",
                         help="Set duration of one client approval")
     parser.add_argument("-H", "--host", help="Set host for client")
     parser.add_argument("-s", "--secret",
@@ -584,14 +576,12 @@
         help="Approve any current client request")
     approve_deny.add_argument("-D", "--deny", action="store_true",
                               help="Deny any current client request")
-    parser.add_argument("--debug", action="store_true",
-                        help="Debug mode (show D-Bus commands)")
     parser.add_argument("--check", action="store_true",
                         help="Run self-test")
     parser.add_argument("client", nargs="*", help="Client name")
 
 
-def commands_from_options(options):
+def commands_and_clients_from_options(options):
 
     commands = []
 
@@ -605,7 +595,7 @@
         commands.append(DisableCmd())
 
     if options.bump_timeout:
-        commands.append(BumpTimeoutCmd())
+        commands.append(BumpTimeoutCmd(options.bump_timeout))
 
     if options.start_checker:
         commands.append(StartCheckerCmd())
@@ -616,8 +606,11 @@
     if options.is_enabled:
         commands.append(IsEnabledCmd())
 
+    if options.remove:
+        commands.append(RemoveCmd())
+
     if options.checker is not None:
-        commands.append(SetCheckerCmd(options.checker))
+        commands.append(SetCheckerCmd())
 
     if options.timeout is not None:
         commands.append(SetTimeoutCmd(options.timeout))
@@ -627,26 +620,26 @@
             SetExtendedTimeoutCmd(options.extended_timeout))
 
     if options.interval is not None:
-        commands.append(SetIntervalCmd(options.interval))
+        command.append(SetIntervalCmd(options.interval))
 
     if options.approved_by_default is not None:
         if options.approved_by_default:
-            commands.append(ApproveByDefaultCmd())
+            command.append(ApproveByDefaultCmd())
         else:
-            commands.append(DenyByDefaultCmd())
+            command.append(DenyByDefaultCmd())
 
     if options.approval_delay is not None:
-        commands.append(SetApprovalDelayCmd(options.approval_delay))
+        command.append(SetApprovalDelayCmd(options.approval_delay))
 
     if options.approval_duration is not None:
-        commands.append(
+        command.append(
             SetApprovalDurationCmd(options.approval_duration))
 
     if options.host is not None:
-        commands.append(SetHostCmd(options.host))
+        command.append(SetHostCmd(options.host))
 
     if options.secret is not None:
-        commands.append(SetSecretCmd(options.secret))
+        command.append(SetSecretCmd(options.secret))
 
     if options.approve:
         commands.append(ApproveCmd())
@@ -654,40 +647,20 @@
     if options.deny:
         commands.append(DenyCmd())
 
-    if options.remove:
-        commands.append(RemoveCmd())
-
     # If no command option has been given, show table of clients,
     # optionally verbosely
     if not commands:
         commands.append(PrintTableCmd(verbose=options.verbose))
 
-    return commands
-
-
-def check_option_syntax(parser, options):
-    """Apply additional restrictions on options, not expressible in
-argparse"""
-
-    def has_actions(options):
-        return any((options.enable,
-                    options.disable,
-                    options.bump_timeout,
-                    options.start_checker,
-                    options.stop_checker,
-                    options.is_enabled,
-                    options.remove,
-                    options.checker is not None,
-                    options.timeout is not None,
-                    options.extended_timeout is not None,
-                    options.interval is not None,
-                    options.approved_by_default is not None,
-                    options.approval_delay is not None,
-                    options.approval_duration is not None,
-                    options.host is not None,
-                    options.secret is not None,
-                    options.approve,
-                    options.deny))
+    return commands, options.client
+
+
+def main():
+    parser = argparse.ArgumentParser()
+
+    add_command_line_options(parser)
+
+    options = parser.parse_args()
 
     if has_actions(options) and not (options.client or options.all):
         parser.error("Options require clients names or --all.")
@@ -700,31 +673,11 @@
         parser.error("--all requires an action.")
     if options.is_enabled and len(options.client) > 1:
         parser.error("--is-enabled requires exactly one client")
-    if options.remove:
-        options.remove = False
-        if has_actions(options) and not options.deny:
-            parser.error("--remove can only be combined with --deny")
-        options.remove = True
-
-
-def main():
-    parser = argparse.ArgumentParser()
-
-    add_command_line_options(parser)
-
-    options = parser.parse_args()
-
-    check_option_syntax(parser, options)
-
-    clientnames = options.client
-
-    if options.debug:
-        log.setLevel(logging.DEBUG)
+
+    commands, clientnames = commands_and_clients_from_options(options)
 
     try:
         bus = dbus.SystemBus()
-        log.debug("D-Bus: Connect to: (name=%r, path=%r)", busname,
-                  server_path)
         mandos_dbus_objc = bus.get_object(busname, server_path)
     except dbus.exceptions.DBusException:
         log.critical("Could not connect to Mandos server")
@@ -743,8 +696,6 @@
     dbus_filter = NullFilter()
     try:
         dbus_logger.addFilter(dbus_filter)
-        log.debug("D-Bus: %s:%s:%s.GetManagedObjects()", busname,
-                  server_path, dbus.OBJECT_MANAGER_IFACE)
         mandos_clients = {path: ifs_and_props[client_interface]
                           for path, ifs_and_props in
                           mandos_serv_object_manager
@@ -762,16 +713,12 @@
     clients = {}
 
     if not clientnames:
-        clients = {(log.debug("D-Bus: Connect to: (name=%r, path=%r)",
-                              busname, str(path)) and False) or
-                   bus.get_object(busname, path): properties
+        clients = {bus.get_object(busname, path): properties
                    for path, properties in mandos_clients.items()}
     else:
         for name in clientnames:
             for path, client in mandos_clients.items():
                 if client["Name"] == name:
-                    log.debug("D-Bus: Connect to: (name=%r, path=%r)",
-                              busname, str(path))
                     client_objc = bus.get_object(busname, path)
                     clients[client_objc] = client
                     break
@@ -780,7 +727,6 @@
                 sys.exit(1)
 
     # Run all commands on clients
-    commands = commands_from_options(options)
     for command in commands:
         command.run(mandos_serv, clients)
 
@@ -800,12 +746,6 @@
 
 class Test_string_to_delta(unittest.TestCase):
     def test_handles_basic_rfc3339(self):
-        self.assertEqual(string_to_delta("PT0S"),
-                         datetime.timedelta())
-        self.assertEqual(string_to_delta("P0D"),
-                         datetime.timedelta())
-        self.assertEqual(string_to_delta("PT1S"),
-                         datetime.timedelta(0, 1))
         self.assertEqual(string_to_delta("PT2H"),
                          datetime.timedelta(0, 7200))
     def test_falls_back_to_pre_1_6_1_with_warning(self):
@@ -846,73 +786,75 @@
                 testcase.assertEqual(dbus_interface,
                                      dbus.PROPERTIES_IFACE)
                 self.attributes[property] = value
+                self.calls.append(("Set", (interface, property, value,
+                                           dbus_interface)))
             def Get(self, interface, property, dbus_interface):
                 testcase.assertEqual(interface, client_interface)
                 testcase.assertEqual(dbus_interface,
                                      dbus.PROPERTIES_IFACE)
+                self.calls.append(("Get", (interface, property,
+                                           dbus_interface)))
                 return self.attributes[property]
-            def Approve(self, approve, dbus_interface):
-                testcase.assertEqual(dbus_interface, client_interface)
-                self.calls.append(("Approve", (approve,
-                                               dbus_interface)))
-        self.client = MockClient(
-            "foo",
-            KeyID=("92ed150794387c03ce684574b1139a65"
-                   "94a34f895daaaf09fd8ea90a27cddb12"),
-            Secret=b"secret",
-            Host="foo.example.org",
-            Enabled=dbus.Boolean(True),
-            Timeout=300000,
-            LastCheckedOK="2019-02-03T00:00:00",
-            Created="2019-01-02T00:00:00",
-            Interval=120000,
-            Fingerprint=("778827225BA7DE539C5A"
-                         "7CFA59CFF7CDBD9A5920"),
-            CheckerRunning=dbus.Boolean(False),
-            LastEnabled="2019-01-03T00:00:00",
-            ApprovalPending=dbus.Boolean(False),
-            ApprovedByDefault=dbus.Boolean(True),
-            LastApprovalRequest="",
-            ApprovalDelay=0,
-            ApprovalDuration=1000,
-            Checker="fping -q -- %(host)s",
-            ExtendedTimeout=900000,
-            Expires="2019-02-04T00:00:00",
-            LastCheckerStatus=0)
-        self.other_client = MockClient(
-            "barbar",
-            KeyID=("0558568eedd67d622f5c83b35a115f79"
-                   "6ab612cff5ad227247e46c2b020f441c"),
-            Secret=b"secretbar",
-            Host="192.0.2.3",
-            Enabled=dbus.Boolean(True),
-            Timeout=300000,
-            LastCheckedOK="2019-02-04T00:00:00",
-            Created="2019-01-03T00:00:00",
-            Interval=120000,
-            Fingerprint=("3E393AEAEFB84C7E89E2"
-                         "F547B3A107558FCA3A27"),
-            CheckerRunning=dbus.Boolean(True),
-            LastEnabled="2019-01-04T00:00:00",
-            ApprovalPending=dbus.Boolean(False),
-            ApprovedByDefault=dbus.Boolean(False),
-            LastApprovalRequest="2019-01-03T00:00:00",
-            ApprovalDelay=30000,
-            ApprovalDuration=1000,
-            Checker=":",
-            ExtendedTimeout=900000,
-            Expires="2019-02-05T00:00:00",
-            LastCheckerStatus=-2)
-        self.clients =  collections.OrderedDict(
-            [
-                (self.client, self.client.attributes),
-                (self.other_client, self.other_client.attributes),
+            def __getitem__(self, key):
+                return self.attributes[key]
+            def __setitem__(self, key, value):
+                self.attributes[key] = value
+        self.clients = collections.OrderedDict([
+            ("foo",
+             MockClient(
+                 "foo",
+                 KeyID=("92ed150794387c03ce684574b1139a65"
+                        "94a34f895daaaf09fd8ea90a27cddb12"),
+                 Secret=b"secret",
+                 Host="foo.example.org",
+                 Enabled=dbus.Boolean(True),
+                 Timeout=300000,
+                 LastCheckedOK="2019-02-03T00:00:00",
+                 Created="2019-01-02T00:00:00",
+                 Interval=120000,
+                 Fingerprint=("778827225BA7DE539C5A"
+                              "7CFA59CFF7CDBD9A5920"),
+                 CheckerRunning=dbus.Boolean(False),
+                 LastEnabled="2019-01-03T00:00:00",
+                 ApprovalPending=dbus.Boolean(False),
+                 ApprovedByDefault=dbus.Boolean(True),
+                 LastApprovalRequest="",
+                 ApprovalDelay=0,
+                 ApprovalDuration=1000,
+                 Checker="fping -q -- %(host)s",
+                 ExtendedTimeout=900000,
+                 Expires="2019-02-04T00:00:00",
+                 LastCheckerStatus=0)),
+            ("barbar",
+             MockClient(
+                 "barbar",
+                 KeyID=("0558568eedd67d622f5c83b35a115f79"
+                        "6ab612cff5ad227247e46c2b020f441c"),
+                 Secret=b"secretbar",
+                 Host="192.0.2.3",
+                 Enabled=dbus.Boolean(True),
+                 Timeout=300000,
+                 LastCheckedOK="2019-02-04T00:00:00",
+                 Created="2019-01-03T00:00:00",
+                 Interval=120000,
+                 Fingerprint=("3E393AEAEFB84C7E89E2"
+                              "F547B3A107558FCA3A27"),
+                 CheckerRunning=dbus.Boolean(True),
+                 LastEnabled="2019-01-04T00:00:00",
+                 ApprovalPending=dbus.Boolean(False),
+                 ApprovedByDefault=dbus.Boolean(False),
+                 LastApprovalRequest="2019-01-03T00:00:00",
+                 ApprovalDelay=30000,
+                 ApprovalDuration=1000,
+                 Checker=":",
+                 ExtendedTimeout=900000,
+                 Expires="2019-02-05T00:00:00",
+                 LastCheckerStatus=-2)),
             ])
-        self.one_client = {self.client: self.client.attributes}
 
 class TestPrintTableCmd(TestCmd):
     def test_normal(self):
-        output = PrintTableCmd().output(self.clients.values())
+        output = PrintTableCmd().output(self.clients)
         expected_output = """
 Name   Enabled Timeout  Last Successful Check
 foo    Yes     00:05:00 2019-02-03T00:00:00  
@@ -920,8 +862,7 @@
 """[1:-1]
         self.assertEqual(output, expected_output)
     def test_verbose(self):
-        output = PrintTableCmd(verbose=True).output(
-            self.clients.values())
+        output = PrintTableCmd(verbose=True).output(self.clients)
         expected_output = """
 Name   Enabled Timeout  Last Successful Check Created             Interval Host            Key ID                                                           Fingerprint                              Check Is Running Last Enabled        Approval Is Pending Approved By Default Last Approval Request Approval Delay Approval Duration Checker              Extended Timeout Expires             Last Checker Status
 foo    Yes     00:05:00 2019-02-03T00:00:00   2019-01-02T00:00:00 00:02:00 foo.example.org 92ed150794387c03ce684574b1139a6594a34f895daaaf09fd8ea90a27cddb12 778827225BA7DE539C5A7CFA59CFF7CDBD9A5920 No               2019-01-03T00:00:00 No                  Yes                                       00:00:00       00:00:01          fping -q -- %(host)s 00:15:00         2019-02-04T00:00:00 0                  
@@ -929,7 +870,7 @@
 """[1:-1]
         self.assertEqual(output, expected_output)
     def test_one_client(self):
-        output = PrintTableCmd().output(self.one_client.values())
+        output = PrintTableCmd().output({"foo": self.clients["foo"]})
         expected_output = """
 Name Enabled Timeout  Last Successful Check
 foo  Yes     00:05:00 2019-02-03T00:00:00  
@@ -993,530 +934,41 @@
         json_data = json.loads(DumpJSONCmd().output(self.clients))
         self.assertDictEqual(json_data, self.expected_json)
     def test_one_client(self):
-        clients = self.one_client
+        clients = {"foo": self.clients["foo"]}
         json_data = json.loads(DumpJSONCmd().output(clients))
         expected_json = {"foo": self.expected_json["foo"]}
         self.assertDictEqual(json_data, expected_json)
 
 class TestIsEnabledCmd(TestCmd):
     def test_is_enabled(self):
-        self.assertTrue(all(IsEnabledCmd().is_enabled(client, properties)
-                            for client, properties in self.clients.items()))
+        self.assertTrue(all(IsEnabledCmd().is_enabled(client)
+                            for client in self.clients.values()))
+    def test_is_enabled_does_get_attribute(self):
+        client = self.clients["foo"]
+        self.assertTrue(IsEnabledCmd().is_enabled(client))
+        self.assertListEqual(client.calls,
+                             [("Get",
+                               ("se.recompile.Mandos.Client",
+                                "Enabled",
+                                "org.freedesktop.DBus.Properties"))])
     def test_is_enabled_run_exits_successfully(self):
+        client = self.clients["foo"]
         with self.assertRaises(SystemExit) as e:
-            IsEnabledCmd().run(None, self.one_client)
+            IsEnabledCmd().run(None, [client])
         if e.exception.code is not None:
             self.assertEqual(e.exception.code, 0)
         else:
             self.assertIsNone(e.exception.code)
     def test_is_enabled_run_exits_with_failure(self):
-        self.client.attributes["Enabled"] = dbus.Boolean(False)
+        client = self.clients["foo"]
+        client["Enabled"] = dbus.Boolean(False)
         with self.assertRaises(SystemExit) as e:
-            IsEnabledCmd().run(None, self.one_client)
+            IsEnabledCmd().run(None, [client])
         if isinstance(e.exception.code, int):
             self.assertNotEqual(e.exception.code, 0)
         else:
             self.assertIsNotNone(e.exception.code)
 
-class TestRemoveCmd(TestCmd):
-    def test_remove(self):
-        class MockMandos(object):
-            def __init__(self):
-                self.calls = []
-            def RemoveClient(self, dbus_path):
-                self.calls.append(("RemoveClient", (dbus_path,)))
-        mandos = MockMandos()
-        super(TestRemoveCmd, self).setUp()
-        RemoveCmd().run(mandos, self.clients)
-        self.assertEqual(len(mandos.calls), 2)
-        for client in self.clients:
-            self.assertIn(("RemoveClient",
-                           (client.__dbus_object_path__,)),
-                          mandos.calls)
-
-class TestApproveCmd(TestCmd):
-    def test_approve(self):
-        ApproveCmd().run(None, self.clients)
-        for client in self.clients:
-            self.assertIn(("Approve", (True, client_interface)),
-                          client.calls)
-
-class TestDenyCmd(TestCmd):
-    def test_deny(self):
-        DenyCmd().run(None, self.clients)
-        for client in self.clients:
-            self.assertIn(("Approve", (False, client_interface)),
-                          client.calls)
-
-class TestEnableCmd(TestCmd):
-    def test_enable(self):
-        for client in self.clients:
-            client.attributes["Enabled"] = False
-
-        EnableCmd().run(None, self.clients)
-
-        for client in self.clients:
-            self.assertTrue(client.attributes["Enabled"])
-
-class TestDisableCmd(TestCmd):
-    def test_disable(self):
-        DisableCmd().run(None, self.clients)
-
-        for client in self.clients:
-            self.assertFalse(client.attributes["Enabled"])
-
-class Unique(object):
-    """Class for objects which exist only to be unique objects, since
-unittest.mock.sentinel only exists in Python 3.3"""
-
-class TestPropertyCmd(TestCmd):
-    """Abstract class for tests of PropertyCmd classes"""
-    def runTest(self):
-        if not hasattr(self, "command"):
-            return
-        values_to_get = getattr(self, "values_to_get",
-                                self.values_to_set)
-        for value_to_set, value_to_get in zip(self.values_to_set,
-                                              values_to_get):
-            for client in self.clients:
-                old_value = client.attributes[self.property]
-                self.assertNotIsInstance(old_value, Unique)
-                client.attributes[self.property] = Unique()
-            self.run_command(value_to_set, self.clients)
-            for client in self.clients:
-                value = client.attributes[self.property]
-                self.assertNotIsInstance(value, Unique)
-                self.assertEqual(value, value_to_get)
-    def run_command(self, value, clients):
-        self.command().run(None, clients)
-
-class TestBumpTimeoutCmd(TestPropertyCmd):
-    command = BumpTimeoutCmd
-    property = "LastCheckedOK"
-    values_to_set = [""]
-
-class TestStartCheckerCmd(TestPropertyCmd):
-    command = StartCheckerCmd
-    property = "CheckerRunning"
-    values_to_set = [dbus.Boolean(True)]
-
-class TestStopCheckerCmd(TestPropertyCmd):
-    command = StopCheckerCmd
-    property = "CheckerRunning"
-    values_to_set = [dbus.Boolean(False)]
-
-class TestApproveByDefaultCmd(TestPropertyCmd):
-    command = ApproveByDefaultCmd
-    property = "ApprovedByDefault"
-    values_to_set = [dbus.Boolean(True)]
-
-class TestDenyByDefaultCmd(TestPropertyCmd):
-    command = DenyByDefaultCmd
-    property = "ApprovedByDefault"
-    values_to_set = [dbus.Boolean(False)]
-
-class TestValueArgumentPropertyCmd(TestPropertyCmd):
-    """Abstract class for tests of PropertyCmd classes using the
-ValueArgumentMixIn"""
-    def runTest(self):
-        if type(self) is TestValueArgumentPropertyCmd:
-            return
-        return super(TestValueArgumentPropertyCmd, self).runTest()
-    def run_command(self, value, clients):
-        self.command(value).run(None, clients)
-
-class TestSetCheckerCmd(TestValueArgumentPropertyCmd):
-    command = SetCheckerCmd
-    property = "Checker"
-    values_to_set = ["", ":", "fping -q -- %s"]
-
-class TestSetHostCmd(TestValueArgumentPropertyCmd):
-    command = SetHostCmd
-    property = "Host"
-    values_to_set = ["192.0.2.3", "foo.example.org"]
-
-class TestSetSecretCmd(TestValueArgumentPropertyCmd):
-    command = SetSecretCmd
-    property = "Secret"
-    values_to_set = [io.BytesIO(b""),
-                     io.BytesIO(b"secret\0xyzzy\nbar")]
-    values_to_get = [b"", b"secret\0xyzzy\nbar"]
-
-class TestSetTimeoutCmd(TestValueArgumentPropertyCmd):
-    command = SetTimeoutCmd
-    property = "Timeout"
-    values_to_set = [datetime.timedelta(),
-                     datetime.timedelta(minutes=5),
-                     datetime.timedelta(seconds=1),
-                     datetime.timedelta(weeks=1),
-                     datetime.timedelta(weeks=52)]
-    values_to_get = [0, 300000, 1000, 604800000, 31449600000]
-
-class TestSetExtendedTimeoutCmd(TestValueArgumentPropertyCmd):
-    command = SetExtendedTimeoutCmd
-    property = "ExtendedTimeout"
-    values_to_set = [datetime.timedelta(),
-                     datetime.timedelta(minutes=5),
-                     datetime.timedelta(seconds=1),
-                     datetime.timedelta(weeks=1),
-                     datetime.timedelta(weeks=52)]
-    values_to_get = [0, 300000, 1000, 604800000, 31449600000]
-
-class TestSetIntervalCmd(TestValueArgumentPropertyCmd):
-    command = SetIntervalCmd
-    property = "Interval"
-    values_to_set = [datetime.timedelta(),
-                     datetime.timedelta(minutes=5),
-                     datetime.timedelta(seconds=1),
-                     datetime.timedelta(weeks=1),
-                     datetime.timedelta(weeks=52)]
-    values_to_get = [0, 300000, 1000, 604800000, 31449600000]
-
-class TestSetApprovalDelayCmd(TestValueArgumentPropertyCmd):
-    command = SetApprovalDelayCmd
-    property = "ApprovalDelay"
-    values_to_set = [datetime.timedelta(),
-                     datetime.timedelta(minutes=5),
-                     datetime.timedelta(seconds=1),
-                     datetime.timedelta(weeks=1),
-                     datetime.timedelta(weeks=52)]
-    values_to_get = [0, 300000, 1000, 604800000, 31449600000]
-
-class TestSetApprovalDurationCmd(TestValueArgumentPropertyCmd):
-    command = SetApprovalDurationCmd
-    property = "ApprovalDuration"
-    values_to_set = [datetime.timedelta(),
-                     datetime.timedelta(minutes=5),
-                     datetime.timedelta(seconds=1),
-                     datetime.timedelta(weeks=1),
-                     datetime.timedelta(weeks=52)]
-    values_to_get = [0, 300000, 1000, 604800000, 31449600000]
-
-class Test_command_from_options(unittest.TestCase):
-    def setUp(self):
-        self.parser = argparse.ArgumentParser()
-        add_command_line_options(self.parser)
-    def assert_command_from_args(self, args, command_cls, **cmd_attrs):
-        """Assert that parsing ARGS should result in an instance of
-COMMAND_CLS with (optionally) all supplied attributes (CMD_ATTRS)."""
-        options = self.parser.parse_args(args)
-        check_option_syntax(self.parser, options)
-        commands = commands_from_options(options)
-        self.assertEqual(len(commands), 1)
-        command = commands[0]
-        self.assertIsInstance(command, command_cls)
-        for key, value in cmd_attrs.items():
-            self.assertEqual(getattr(command, key), value)
-    def test_print_table(self):
-        self.assert_command_from_args([], PrintTableCmd,
-                                      verbose=False)
-
-    def test_print_table_verbose(self):
-        self.assert_command_from_args(["--verbose"], PrintTableCmd,
-                                      verbose=True)
-
-    def test_print_table_verbose_short(self):
-        self.assert_command_from_args(["-v"], PrintTableCmd,
-                                      verbose=True)
-
-    def test_enable(self):
-        self.assert_command_from_args(["--enable", "foo"], EnableCmd)
-
-    def test_enable_short(self):
-        self.assert_command_from_args(["-e", "foo"], EnableCmd)
-
-    def test_disable(self):
-        self.assert_command_from_args(["--disable", "foo"],
-                                      DisableCmd)
-
-    def test_disable_short(self):
-        self.assert_command_from_args(["-d", "foo"], DisableCmd)
-
-    def test_bump_timeout(self):
-        self.assert_command_from_args(["--bump-timeout", "foo"],
-                                      BumpTimeoutCmd)
-
-    def test_bump_timeout_short(self):
-        self.assert_command_from_args(["-b", "foo"], BumpTimeoutCmd)
-
-    def test_start_checker(self):
-        self.assert_command_from_args(["--start-checker", "foo"],
-                                      StartCheckerCmd)
-
-    def test_stop_checker(self):
-        self.assert_command_from_args(["--stop-checker", "foo"],
-                                      StopCheckerCmd)
-
-    def test_remove(self):
-        self.assert_command_from_args(["--remove", "foo"],
-                                      RemoveCmd)
-
-    def test_remove_short(self):
-        self.assert_command_from_args(["-r", "foo"], RemoveCmd)
-
-    def test_checker(self):
-        self.assert_command_from_args(["--checker", ":", "foo"],
-                                      SetCheckerCmd, value_to_set=":")
-
-    def test_checker_empty(self):
-        self.assert_command_from_args(["--checker", "", "foo"],
-                                      SetCheckerCmd, value_to_set="")
-
-    def test_checker_short(self):
-        self.assert_command_from_args(["-c", ":", "foo"],
-                                      SetCheckerCmd, value_to_set=":")
-
-    def test_timeout(self):
-        self.assert_command_from_args(["--timeout", "PT5M", "foo"],
-                                      SetTimeoutCmd,
-                                      value_to_set=300000)
-
-    def test_timeout_short(self):
-        self.assert_command_from_args(["-t", "PT5M", "foo"],
-                                      SetTimeoutCmd,
-                                      value_to_set=300000)
-
-    def test_extended_timeout(self):
-        self.assert_command_from_args(["--extended-timeout", "PT15M",
-                                       "foo"],
-                                      SetExtendedTimeoutCmd,
-                                      value_to_set=900000)
-
-    def test_interval(self):
-        self.assert_command_from_args(["--interval", "PT2M", "foo"],
-                                      SetIntervalCmd,
-                                      value_to_set=120000)
-
-    def test_interval_short(self):
-        self.assert_command_from_args(["-i", "PT2M", "foo"],
-                                      SetIntervalCmd,
-                                      value_to_set=120000)
-
-    def test_approve_by_default(self):
-        self.assert_command_from_args(["--approve-by-default", "foo"],
-                                      ApproveByDefaultCmd)
-
-    def test_deny_by_default(self):
-        self.assert_command_from_args(["--deny-by-default", "foo"],
-                                      DenyByDefaultCmd)
-
-    def test_approval_delay(self):
-        self.assert_command_from_args(["--approval-delay", "PT30S",
-                                       "foo"], SetApprovalDelayCmd,
-                                      value_to_set=30000)
-
-    def test_approval_duration(self):
-        self.assert_command_from_args(["--approval-duration", "PT1S",
-                                       "foo"], SetApprovalDurationCmd,
-                                      value_to_set=1000)
-
-    def test_host(self):
-        self.assert_command_from_args(["--host", "foo.example.org",
-                                       "foo"], SetHostCmd,
-                                      value_to_set="foo.example.org")
-
-    def test_host_short(self):
-        self.assert_command_from_args(["-H", "foo.example.org",
-                                       "foo"], SetHostCmd,
-                                      value_to_set="foo.example.org")
-
-    def test_secret_devnull(self):
-        self.assert_command_from_args(["--secret", os.path.devnull,
-                                       "foo"], SetSecretCmd,
-                                      value_to_set=b"")
-
-    def test_secret_tempfile(self):
-        with tempfile.NamedTemporaryFile(mode="r+b") as f:
-            value = b"secret\0xyzzy\nbar"
-            f.write(value)
-            f.seek(0)
-            self.assert_command_from_args(["--secret", f.name,
-                                           "foo"], SetSecretCmd,
-                                          value_to_set=value)
-
-    def test_secret_devnull_short(self):
-        self.assert_command_from_args(["-s", os.path.devnull, "foo"],
-                                      SetSecretCmd, value_to_set=b"")
-
-    def test_secret_tempfile_short(self):
-        with tempfile.NamedTemporaryFile(mode="r+b") as f:
-            value = b"secret\0xyzzy\nbar"
-            f.write(value)
-            f.seek(0)
-            self.assert_command_from_args(["-s", f.name, "foo"],
-                                          SetSecretCmd,
-                                          value_to_set=value)
-
-    def test_approve(self):
-        self.assert_command_from_args(["--approve", "foo"],
-                                      ApproveCmd)
-
-    def test_approve_short(self):
-        self.assert_command_from_args(["-A", "foo"], ApproveCmd)
-
-    def test_deny(self):
-        self.assert_command_from_args(["--deny", "foo"], DenyCmd)
-
-    def test_deny_short(self):
-        self.assert_command_from_args(["-D", "foo"], DenyCmd)
-
-    def test_dump_json(self):
-        self.assert_command_from_args(["--dump-json"], DumpJSONCmd)
-
-    def test_is_enabled(self):
-        self.assert_command_from_args(["--is-enabled", "foo"],
-                                      IsEnabledCmd)
-
-    def test_is_enabled_short(self):
-        self.assert_command_from_args(["-V", "foo"], IsEnabledCmd)
-
-    def test_deny_before_remove(self):
-        options = self.parser.parse_args(["--deny", "--remove", "foo"])
-        check_option_syntax(self.parser, options)
-        commands = commands_from_options(options)
-        self.assertEqual(len(commands), 2)
-        self.assertIsInstance(commands[0], DenyCmd)
-        self.assertIsInstance(commands[1], RemoveCmd)
-
-    def test_deny_before_remove_reversed(self):
-        options = self.parser.parse_args(["--remove", "--deny", "--all"])
-        check_option_syntax(self.parser, options)
-        commands = commands_from_options(options)
-        self.assertEqual(len(commands), 2)
-        self.assertIsInstance(commands[0], DenyCmd)
-        self.assertIsInstance(commands[1], RemoveCmd)
-
-
-class Test_check_option_syntax(unittest.TestCase):
-    # This mostly corresponds to the definition from has_actions() in
-    # check_option_syntax()
-    actions = {
-        # The actual values set here are not that important, but we do
-        # at least stick to the correct types, even though they are
-        # never used
-        "enable": True,
-        "disable": True,
-        "bump_timeout": True,
-        "start_checker": True,
-        "stop_checker": True,
-        "is_enabled": True,
-        "remove": True,
-        "checker": "x",
-        "timeout": datetime.timedelta(),
-        "extended_timeout": datetime.timedelta(),
-        "interval": datetime.timedelta(),
-        "approved_by_default": True,
-        "approval_delay": datetime.timedelta(),
-        "approval_duration": datetime.timedelta(),
-        "host": "x",
-        "secret": io.BytesIO(b"x"),
-        "approve": True,
-        "deny": True,
-    }
-
-    def setUp(self):
-        self.parser = argparse.ArgumentParser()
-        add_command_line_options(self.parser)
-
-    @contextlib.contextmanager
-    def assertParseError(self):
-        with self.assertRaises(SystemExit) as e:
-            with self.temporarily_suppress_stderr():
-                yield
-        # Exit code from argparse is guaranteed to be "2".  Reference:
-        # https://docs.python.org/3/library/argparse.html#exiting-methods
-        self.assertEqual(e.exception.code, 2)
-
-    @staticmethod
-    @contextlib.contextmanager
-    def temporarily_suppress_stderr():
-        null = os.open(os.path.devnull, os.O_RDWR)
-        stderrcopy = os.dup(sys.stderr.fileno())
-        os.dup2(null, sys.stderr.fileno())
-        os.close(null)
-        try:
-            yield
-        finally:
-            # restore stderr
-            os.dup2(stderrcopy, sys.stderr.fileno())
-            os.close(stderrcopy)
-
-    def check_option_syntax(self, options):
-        check_option_syntax(self.parser, options)
-
-    def test_actions_requires_client_or_all(self):
-        for action, value in self.actions.items():
-            options = self.parser.parse_args()
-            setattr(options, action, value)
-            with self.assertParseError():
-                self.check_option_syntax(options)
-
-    def test_actions_conflicts_with_verbose(self):
-        for action, value in self.actions.items():
-            options = self.parser.parse_args()
-            setattr(options, action, value)
-            options.verbose = True
-            with self.assertParseError():
-                self.check_option_syntax(options)
-
-    def test_dump_json_conflicts_with_verbose(self):
-        options = self.parser.parse_args()
-        options.dump_json = True
-        options.verbose = True
-        with self.assertParseError():
-            self.check_option_syntax(options)
-
-    def test_dump_json_conflicts_with_action(self):
-        for action, value in self.actions.items():
-            options = self.parser.parse_args()
-            setattr(options, action, value)
-            options.dump_json = True
-            with self.assertParseError():
-                self.check_option_syntax(options)
-
-    def test_all_can_not_be_alone(self):
-        options = self.parser.parse_args()
-        options.all = True
-        with self.assertParseError():
-            self.check_option_syntax(options)
-
-    def test_all_is_ok_with_any_action(self):
-        for action, value in self.actions.items():
-            options = self.parser.parse_args()
-            setattr(options, action, value)
-            options.all = True
-            self.check_option_syntax(options)
-
-    def test_is_enabled_fails_without_client(self):
-        options = self.parser.parse_args()
-        options.is_enabled = True
-        with self.assertParseError():
-            self.check_option_syntax(options)
-
-    def test_is_enabled_works_with_one_client(self):
-        options = self.parser.parse_args()
-        options.is_enabled = True
-        options.client = ["foo"]
-        self.check_option_syntax(options)
-
-    def test_is_enabled_fails_with_two_clients(self):
-        options = self.parser.parse_args()
-        options.is_enabled = True
-        options.client = ["foo", "barbar"]
-        with self.assertParseError():
-            self.check_option_syntax(options)
-
-    def test_remove_can_only_be_combined_with_action_deny(self):
-        for action, value in self.actions.items():
-            if action in {"remove", "deny"}:
-                continue
-            options = self.parser.parse_args()
-            setattr(options, action, value)
-            options.all = True
-            options.remove = True
-            with self.assertParseError():
-                self.check_option_syntax(options)
-
 
 
 def should_only_run_tests():

=== modified file 'mandos-ctl.xml'
--- mandos-ctl.xml	2019-03-08 23:55:34 +0000
+++ mandos-ctl.xml	2019-02-10 04:20:26 +0000
@@ -2,7 +2,7 @@
 <!DOCTYPE refentry PUBLIC "-//OASIS//DTD DocBook XML V4.5//EN"
 	"http://www.oasis-open.org/docbook/xml/4.5/docbookx.dtd" [
 <!ENTITY COMMANDNAME "mandos-ctl">
-<!ENTITY TIMESTAMP "2019-03-09">
+<!ENTITY TIMESTAMP "2019-02-10">
 <!ENTITY % common SYSTEM "common.ent">
 %common;
 ]>
@@ -62,22 +62,6 @@
   <refsynopsisdiv>
     <cmdsynopsis>
       <command>&COMMANDNAME;</command>
-      <group>
-	  <arg choice="plain"><option>--verbose</option></arg>
-	  <arg choice="plain"><option>-v</option></arg>
-	  <sbr/>
-	  <arg choice="plain"><option>--dump-json</option></arg>
-	  <arg choice="plain"><option>-j</option></arg>
-      </group>
-      <arg><option>--debug</option></arg>
-      <group>
-	<arg rep='repeat' choice='plain'>
-	  <replaceable>CLIENT</replaceable>
-	</arg>
-      </group>
-    </cmdsynopsis>
-    <cmdsynopsis>
-      <command>&COMMANDNAME;</command>
       <group choice="req">
 	<group>
 	  <arg choice="plain"><option>--enable</option></arg>
@@ -94,10 +78,18 @@
 	<sbr/>
 	<group>
 	  <arg choice="plain"><option>--start-checker</option></arg>
+	</group>
+	<sbr/>
+	<group>
 	  <arg choice="plain"><option>--stop-checker</option></arg>
 	</group>
 	<sbr/>
 	<group>
+	  <arg choice="plain"><option>--remove</option></arg>
+	  <arg choice="plain"><option>-r</option></arg>
+	</group>
+	<sbr/>
+	<group>
 	  <arg choice="plain"><option>--checker
 	  <replaceable>COMMAND</replaceable></option></arg>
 	  <arg choice="plain"><option>-c
@@ -141,6 +133,13 @@
 	</group>
 	<sbr/>
 	<group>
+	  <arg choice="plain"><option>--interval
+	  <replaceable>TIME</replaceable></option></arg>
+	  <arg choice="plain"><option>-i
+	  <replaceable>TIME</replaceable></option></arg>
+	</group>
+	<sbr/>
+	<group>
 	  <arg choice="plain"><option>--host
 	  <replaceable>STRING</replaceable></option></arg>
 	  <arg choice="plain"><option>-H
@@ -163,7 +162,6 @@
 	</group>
       </group>
       <sbr/>
-      <arg><option>--debug</option></arg>
       <group choice="req">
 	<arg choice="plain"><option>--all</option></arg>
 	<arg choice="plain"><option>-a</option></arg>
@@ -175,18 +173,13 @@
     <cmdsynopsis>
       <command>&COMMANDNAME;</command>
       <group>
-	<arg choice="plain"><option>--deny</option></arg>
-	<arg choice="plain"><option>-D</option></arg>
-      </group>
-      <group choice="req">
-	  <arg choice="plain"><option>--remove</option></arg>
-	  <arg choice="plain"><option>-r</option></arg>
-      </group>
-      <sbr/>
-      <arg><option>--debug</option></arg>
-      <group choice="req">
-	<arg choice="plain"><option>--all</option></arg>
-	<arg choice="plain"><option>-a</option></arg>
+	  <arg choice="plain"><option>--verbose</option></arg>
+	  <arg choice="plain"><option>-v</option></arg>
+	  <sbr/>
+	  <arg choice="plain"><option>--dump-json</option></arg>
+	  <arg choice="plain"><option>-j</option></arg>
+      </group>
+      <group>
 	<arg rep='repeat' choice='plain'>
 	  <replaceable>CLIENT</replaceable>
 	</arg>
@@ -198,7 +191,6 @@
 	<arg choice="plain"><option>--is-enabled</option></arg>
 	<arg choice="plain"><option>-V</option></arg>
       </group>
-      <arg><option>--debug</option></arg>
       <arg choice='plain'><replaceable>CLIENT</replaceable></arg>
     </cmdsynopsis>
     <cmdsynopsis>
@@ -512,15 +504,6 @@
       </varlistentry>
       
       <varlistentry>
-	<term><option>--debug</option></term>
-	<listitem>
-	  <para>
-	    Show debug output; currently, this means show D-Bus calls.
-	  </para>
-	</listitem>
-      </varlistentry>
-      
-      <varlistentry>
 	<term><option>--check</option></term>
 	<listitem>
 	  <para>
@@ -598,7 +581,7 @@
       <para>
 
 <!-- do not wrap this line -->
-<userinput>&COMMANDNAME; --timeout="PT5M" --interval="PT1M" foo1.example.org foo2.example.org</userinput>
+<userinput>&COMMANDNAME; --timeout="5m" --interval="1m" foo1.example.org foo2.example.org</userinput>
 
       </para>
     </informalexample>

